<?php
class CDb extends CObject {
	
	var $filename = null;
	var $data = array(
		'index' => 1,
		'rows' => array(),
	);
	var $fields = array();
	var $updated = true;
	var $order = null;
	var $direction = null;
	var $insertId = null;
	
	
	function getPath() {
		return dirname(dirname(__FILE__)) . '/dbdata/' . $this->filename;
	}
	
	
	function load() {
		if ($this->updated) {
			$path = $this->getPath();
			if (file_exists($path)) {
				$contents = file_get_contents($path);
				if ($contents) {
					$this->data = unserialize(file_get_contents($path));
					$this->updated = false;
				}
			}
		}
	}
	
	
	function save() {
		if ($fp = fopen($this->getPath(), 'w')) {
			flock($fp, LOCK_EX);
			fwrite($fp, serialize($this->data));
			fclose($fp);
			$this->updated = true;
			return true;
		}
		return false;
	}
	
	
	function insert($data) {
		$this->load();
		$save = array();
		foreach ($data as $key => $value) {
			if (in_array($key, $this->fields)) {
				$save[$key] = $value;
			}
		}
		$save['id'] = $this->data['index'];
		$this->data['index']++;
		$this->data['rows'][] = $save;
		$this->insertId = $save['id'];
		$this->save();
	}
	
	
	function getInsertId() {
		return $this->insertId;
	}
	
	
	function update($data) {
		$this->load();
		foreach ($this->data['rows'] as $i => $row) {
			if ($row['id'] == $data['id']) {
				foreach ($data as $key => $value) {
					if (in_array($key, $this->fields)) {
						$row[$key] = $value;
					}
				}
				$this->data['rows'][$i] = $row;
				break;
			}
		}
		$this->save();
	}
	
	
	function del($id) {
		$this->load();
		foreach ($this->data['rows'] as $i => $row) {
			if ($row['id'] == $id) {
				$this->data['rows'] = array_merge(array_slice($this->data['rows'], 0, $i), array_slice($this->data['rows'], $i + 1));
				break;
			}
		}
		$this->save();
	}
	
	
	function read($id) {
		$this->load();
		foreach ($this->data['rows'] as $row) {
			if ($row['id'] == $id) return $row;
		}
		return null;
	}
	
	
	function findAll($conditions = null, $order = null, $limit = null) {
		$this->load();
		if (!$conditions) {
			$results = $this->data['rows'];
		} else {
			$results = array();
			foreach ($this->data['rows'] as $row) {
				$match = true;
				foreach ($conditions as $key => $value) {
					if ($row[$key] != $value) {
						$match = false;
						break;
					}
				}
				if ($match) {
					$results[] = $row;
				}
			}
		}
		if ($order) {
			$parts = explode(' ', $order);
			$this->order = $parts[0];
			$this->direction = $parts[1] ? 1 : -1;
			usort($results, array(&$this, 'cmp'));
		}
		if ($limit) {
			$parts = explode(',', $limit);
			if ($parts[1]) {
				$start = $parts[0];
				$number = $parts[1];
			} else {
				$start = 0;
				$number = $limit;
			}
			$results = array_slice($results, $start, $number);
		}
		return $results;
	}
	
	
	function cmp($a, $b) {
		if ($a[$this->order] == $b[$this->order]) return 0;
		return $a[$this->order] < $b[$this->order] ? $this->direction : (-1 * $this->direction);
	}
	
	
	function find($conditions = null, $order = null) {
		$results = $this->findAll($conditions, $order);
		if ($results) return $results[0];
		return null;
	}
	
	
	function deleteAll($conditions) {
		$this->load();
		if (!$conditions) return;
		$save = $this->data['rows'];
		$this->data['rows'] = array();
		foreach ($save as $row) {
			$match = true;
			foreach ($conditions as $key => $value) {
				if ($row[$key] != $value) {
					$match = false;
					break;
				}
			}
			if (!$match) {
				$this->data['rows'][] = $row;
			}
		}
		$this->save();
	}
	
	
	function truncate() {
		if ($fp = fopen($this->getPath(), 'w')) {
			flock($fp, LOCK_EX);
			fclose($fp);
			$this->updated = true;
			return true;
		}
		return false;
	}
}
